#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

require 'TT_Lib2/core.rb'

# ...
#
# @deprecated Unfinished
# @since 2.1.0
module TT::Gizmo
  
  # @deprecated Unfinished
	# @since 2.5.0
  class Axis
    
    #attr_accessor(:origin, :normal)
    attr_accessor(:origin, :vector)
    
    def initialize(origin, vector, color, active_color)
      @origin = origin.clone
      @vector = vector.clone
      @color = color#.clone (!) .clone returns nil
      @active_color = active_color
      
      # Cache of the axis origin and orientation. Cached on onLButtonDown
      @old_origin = nil
      @old_vector = nil
      @old_axis = nil # [pt1, pt2]
      
      @pt_start = nil # Startpoint - IP projected to selected axis
      @pt_screen_start = nil # Screen projection of @pt_start
      
      @pt_mouse = nil # Mouse Point3d - projected to selected axis
      @pt_screen_mouse = nil

      @mouse_vector = nil # DEBUG
      
      # User InputPoint
      @ip = Sketchup::InputPoint.new
      
      @selected = false
      @interacting = false
      
      # Contains a Symbol when the user is interacting with one of the axis'
      # interactive areas. The Symbol indicate which part it is.
      # Possible Values:
      # :move
      # :rotate
      # :scale
      # :axis
      @action = nil
      
      # Event callbacks
      @callback = nil
      @callback_start = nil
      @callback_end = nil
      
      update_segments( Sketchup.active_model.active_view )
    end
    
    def on_transform(&block)
      @callback = block
    end
    
    def on_transform_start(&block)
      @callback_start = block
    end
    
    def on_transform_end(&block)
      @callback_end = block
    end
    
    def selected?
      @selected
    end
    
    def origin=(value)
      @origin = value.clone
    end
    
    # Check if the mouse is interacting with any of the segments.
    def onLButtonDown(flags, x, y, view)
      if @selected
        @interacting = true
        
        @old_origin = @origin.clone
        
        # Get input point closest to the selected axis
        ip = view.inputpoint(x,y)
        @pt_start = ip.position.project_to_line( [@origin, @vector] )
        
        # Project to screen axis
        @old_axis = [ @origin, @origin.offset(@vector, 10) ]
        screen_axis = screen_points( @old_axis, view )
        @pt_screen_start = Geom::Point3d.new(x,y,0).project_to_line( screen_axis )
        
        @callback_start.call(self) unless @callback_start.nil?
        true
      else
        false
      end
    end
    
    def onLButtonUp(flags, x, y, view)
      if @interacting
        @ip.clear
        @callback_end.call(self) unless @callback_end.nil?
        @interacting = false
        true
      else
        false
      end
    end
    
    # Check if the mouse is interacting with any of the segments.
    def onMouseMove(flags, x, y, view)
      if @interacting
        case @action
        when :move
          action_move(x, y, view) # return true
        else
          true # Prevent event bubble
        end
      else
        # Highlight axis segment if mouse hovers over it.
        highlight_segment(x, y, view)
      end
    end
    
    def action_move(x, y, view)
      # Axis vector
      vector = @old_axis[0].vector_to( @old_axis[1] )
      
      # Get mouse point on selected axis
      @ip.pick(view, x, y)
      @pt_mouse = @ip.position.project_to_line( @old_axis )

      # Get axis in screen space
      screen_axis = screen_points( @old_axis, view )
      
      # Calculate the Screen offset distance
      @pt_screen_mouse = Geom::Point3d.new(x,y,0).project_to_line( screen_axis )
      mouse_distance = @pt_screen_start.distance( @pt_screen_mouse )
      
      if mouse_distance > 0
        # Direction vector
        direction = vector.clone
        
        # Get movement vector in screen space
        v = @pt_screen_start.vector_to( @pt_screen_mouse )
        
        # (!) validate vector.
        screen_v = screen_axis[0].vector_to( screen_axis[1] )
        direction.reverse! unless screen_v.samedirection?( v )
        
        # Work out how much in real world distance the offset is.
        screen_distance = v.length
        world_distance = view.pixels_to_model( screen_distance, @old_origin )
        
        Sketchup.vcb_label = 'Distance'
        Sketchup.vcb_value = world_distance.to_l.to_s
        view.tooltip = "Move: #{world_distance.to_l.to_s}"
        
        @mouse_vector = v.clone # DEBUG
        
        # Offset Origin
        offset = @old_origin.offset( direction, world_distance )
        
        # Global Offset Vectors
        v_step  = @origin.vector_to(offset)
        v_total = @old_origin.vector_to(offset)

        # Move Gizmo Origin
        t_total = Geom::Transformation.new( v_total )
        @origin = @old_origin.transform( t_total )

        update_segments(view)
        
        # Call event with local transformations
        t_step  = Geom::Transformation.new( v_step )
        t_total = Geom::Transformation.new( v_total )
        @callback.call( self, t_step, t_total ) unless @callback.nil?
      end
      true
    end
    
    def highlight_segment(x, y, view)
      # Highlight axis segment if mouse hovers over it.
      ph = view.pick_helper
      ph.do_pick( x,y )
      @selected = false
      @action = false
      @segments.each { |action, segments|
        segments.each { |segment|
          if ph.pick_segment( segment, x, y )
            @selected = true
            @action = action
            # store mouse position
            view.invalidate
            return true
          end
        }
      }
      false
    end
    
    def update_segments(view)
      @segments = {}
      # Arm
      arrow_offset = view.pixels_to_model(110, @origin)
      @segments[:arm] = []
      @segments[:arm] << [ ORIGIN, ORIGIN.offset(Z_AXIS, arrow_offset) ]
      # Move Arrow
      axis_length = view.pixels_to_model(150, @origin)
      radius = view.pixels_to_model(10, @origin)
      arrow_origin = ORIGIN.offset(Z_AXIS, arrow_offset)
      arrow_end = @segments[:arm][0][1]
      @segments[:move] = []
      arrow_circle = TT::Geom3d.circle( arrow_origin, Z_AXIS, radius, 8 )
      arrow_end = ORIGIN.offset(Z_AXIS, axis_length)
      @segments[:move] << arrow_circle
      arrow_circle.each { |pt|
        @segments[:move] << [pt, arrow_end]
      }
      # Rotate Circle
      rotate_radius = view.pixels_to_model(10, @origin)
      rotate_offset = view.pixels_to_model(75, @origin)
      rotate_origin = ORIGIN.offset(Z_AXIS, rotate_offset)
      @segments[:rotate] = []
      rotate_circle = TT::Geom3d.circle( rotate_origin, Z_AXIS, rotate_radius, 16 )
      @segments[:rotate] << rotate_circle
      # Transform to world space
      t = Geom::Transformation.new( @origin, @vector )
      @segments.each { |key, segments|
        segments.each { |segment|
          segment.map! { |pt| pt.transform(t) }
        }
      }
      nil
    end
    
    def draw(view)
      update_segments( view )
      if @segments
        view.line_stipple = ''
        view.line_width = 2
        @segments.each { |key, segments|
          segments.each { |segment|
            view.drawing_color = (@selected && @action == key) ? @active_color : @color
            #view.draw( GL_LINE_LOOP, segment )
            view.draw2d( GL_LINE_LOOP, segment.map { |pt| view.screen_coords(pt) } )
          }
          # Draw Arrow Fill
          # (!) Only if SU8M1B6+
          circle = @segments[:move][0]
          circle << circle.first
          pt = @segments[:move][1].last
          (0..circle.size-2).each { |i|
            pts = [
              circle[i],
              circle[i+1],
              pt
            ]
            #color = @color.clone # (!) BugSplats!
            color = TT::Color.clone( @color )
            color.alpha = 45
            view.drawing_color = color
            view.draw2d( GL_TRIANGLES, pts.map { |pt| view.screen_coords(pt) } )
          }
        }
        # <Debug>
        #if @ip.valid?
        #  @ip.draw( view )
        #  # Mouse Ticks
        #  view.drawing_color = 'red'
        #  view.line_stipple = ''
        #  view.draw2d( GL_LINES, [ @pt_screen_mouse, @pt_screen_mouse.offset(Y_AXIS.reverse,10) ] )
        #  view.draw2d( GL_LINES, [ @pt_screen_start, @pt_screen_start.offset(Y_AXIS.reverse,10) ] )
        #  # Direction from @old_origin
        #  view.drawing_color = 'pink'
        #  opt = view.screen_coords( @origin )
        #  view.draw2d( GL_LINES, [ opt, opt.offset(@mouse_vector,50) ] )
        #end
        # </Debug>
      end
    end
    
    def screen_points( points, view )
      points.map { |pt|
        screen_pt = view.screen_coords(pt)
        screen_pt.z = 0
        screen_pt
      }
    end
    
  end # class Axis
  
  
  # @deprecated Unfinished
	# @since 2.1.0
  class Manipulator
    
    CLR_X_AXIS    = Sketchup::Color.new( 255,   0,   0 )
    CLR_Y_AXIS    = Sketchup::Color.new(   0, 128,   0 )
    CLR_Z_AXIS    = Sketchup::Color.new(   0,   0, 255 )
    CLR_SELECTED  = Sketchup::Color.new( 255, 255,   0 )
    
    # (?) reader?
    attr_accessor(:origin)
    
    def initialize(origin, xaxis, yaxis, zaxis)
      @origin = origin
      @v_xaxis = xaxis
      @v_yaxis = yaxis
      @v_zaxis = zaxis
      
      @axes = []
      @axes << TT::Gizmo::Axis.new( @origin, xaxis, CLR_X_AXIS, CLR_SELECTED )
      @axes << TT::Gizmo::Axis.new( @origin, yaxis, CLR_Y_AXIS, CLR_SELECTED )
      @axes << TT::Gizmo::Axis.new( @origin, zaxis, CLR_Z_AXIS, CLR_SELECTED )
      
      @active_axis = nil
      
      @axes.each { |axis|
        axis.on_transform_start { |axis|
          @callback_start.call unless @callback_start.nil?
        }
        axis.on_transform_end { |axis|
          @callback_end.call unless @callback_end.nil?
        }
        axis.on_transform { |axis, t_step, t_total|
          @origin = axis.origin
          update_axes( axis )
          @callback.call(t_step, t_total) unless @callback.nil?
        }
      }
      
      # Event callbacks
      @callback = nil
      @callback_start = nil
      @callback_end = nil
      
      #update()
    end
    
    def normal
      @axes.z.vector
    end
    
    def resume(view)
      #update()
    end
    
    def on_transform(&block)
      @callback = block
    end
    
    def on_transform_start(&block)
      @callback_start = block
    end
    
    def on_transform_end(&block)
      @callback_end = block
    end
    
    def origin=(value)
      @origin = value
      @axes.each { |axis|
        axis.origin = value
      }
      #update()
    end
    
    def onMouseMove(flags, x, y, view)
      if @active_axis
        @active_axis.onMouseMove(flags, x, y, view)
        return true
      else
        @axes.each { |axis|
          return true if axis.onMouseMove(flags, x, y, view)
        }
      end
      false
    end
        
    def onLButtonDown(flags, x, y, view)
      @axes.each { |axis|
        if axis.onLButtonDown(flags, x, y, view)
          @active_axis = axis
          return true 
        end
      }
      false
    end
    
    def onLButtonUp(flags, x, y, view)
      @axes.each { |axis|
        if axis.onLButtonUp(flags, x, y, view)
          @active_axis = nil
          return true
        end
      }
      false
    end
    
    def draw(view)
      @axes.each { |axis|
        axis.draw( view )
      }
    end
    
    private
    
    def update_axes(axis)
      @axes.each { |a|
        next if axis == a
        a.origin = axis.origin
      }
    end
    
  end # class Manipulator
  
end # module TT::Gizmo