#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

require 'TT_Lib2/core.rb'

# Wrapper library for creating and manipulating WebDialogs and HTML content
# via Ruby.
#
# @deprecated Alpha stage. Very likely to be subject to change!
module TT::GUI

  # http://juixe.com/techknow/index.php/2007/01/22/ruby-class-tutorial/
  # http://stackoverflow.com/questions/1645398/ruby-include-question
	
  # All GUI elements inherit this class.
  #
	# @abstract
	# @since 2.4.0
	class Control
		attr_accessor( :ui_id ) # @since 2.4.0
		attr_accessor( :left, :right, :top, :bottom, :height, :width ) # @since 2.4.0
		attr_accessor( :parent, :window ) # @since 2.4.0
		attr_accessor( :tooltip ) # @since 2.5.0
		
		# @since 2.4.0
		def initialize(*args)
      # Unique identifier used in the WebDialog's HTML.
			@ui_id = 'UI_' + self.object_id.to_s
      # Hash with Symbols for keys idenitfying the event.
      # Each event is an array of Proc's.
      @events = {}
		end
    
    # (?) Private?
    # Adds an event to the stack.
    #
    # @param [Symbol] event
    # @param [Proc] proc
    #
    # @return [nil]
		# @since 2.5.0
    def add_event(event, proc)
      @events[event] ||= []
      @events[event] << proc
      nil
    end
    
    # Triggers the given event. All attached procs for that event will be called.
    #
    # @param [Symbol] event
    # @param [Array] args Set of arguments to be passed to the called procs.
    #
    # @return [Boolean]
		# @since 2.5.0
    def call_event(event, args = nil)
      TT::debug "call_event(#{event.to_s})"
      if @events.key?( event )
        @events[event].each { |proc|
          if args.nil?
            proc.call
          else
            proc.call(*args)
          end
        }
        true
      else
        # (?) Raise error?
        false
      end
    end
		
		# @since 2.4.0
		def move(left, top)
			@left = left
			@top  = top
      # (!) Update webdialog
		end
		
		# @since 2.4.0
		def size(width, height)
			@width  = width
			@height = height
      # (!) Update webdialog
		end
		
		# @since 2.4.0
		def position(top, right, bottom, left)
			@top	= top
			@right	= right
			@bottom	= bottom
			@left	= left
      # (!) Update webdialog
		end
		
    # @deprecated Early test method.
		# @since 2.4.0
		def css
			props = {
				:left	=> @left,
				:top	=> @top,
				:right	=> @right,
				:bottom	=> @bottom,
				:width	=> @width,
				:height	=> @height
			}
			
			str = ''
			
			props.each { |key, value|
				next if value.nil?
				value = value.to_s + 'px' if value.is_a?(Fixnum)
				str += "#{key}:#{value};"
			}
			
			return str if @left.nil? && @top.nil? && @right.nil? && @bottom.nil?
			return 'position:absolute;' + str
		end
    
    # (!) Need explicit :position property.
		# @since 2.5.0
    def positioned?
      ( @left || @right || @top || @bottom ) ? true : false
    end
    
		# @since 2.5.0
    def properties
      options = TT::JSON.new
      options['id'] = @ui_id
      options['parent'] = @parent.ui_id if @parent.is_a?( Control )
      options['type'] = self.class.name
      if positioned?
        options['top']    = @top    if @top
        options['bottom'] = @bottom if @bottom
        options['left']   = @left   if @left
        options['right']  = @right  if @right
      end
      options['width']    = @width  if @width
      options['height']   = @height if @height
      if self.respond_to?( :custom_properties )
        options.merge!( custom_properties() )
      end
      options
    end
	end

	
	# @abstract +Container+ and +Window+ implements this.
	# @since 2.4.0
	module ContainerElement
		# @since 2.4.0
		attr :child
		
		# @since 2.4.0
		def initialize(*args)
			super(*args)
			@child = []
		end
		
		# @since 2.4.0
		def add_control(control)
			throw :invalid_control unless control.is_a?(Control)
			@child << control
			control.parent = self
			control.window = self.window
      
      # Add to Webdialog
      if @window.visible?
        @window.add_control_to_webdialog(control)
      end
		end
		
		# @since 2.4.0
		def to_html
			style = ( (css = self.css).nil? ) ? '' : " style='#{css}'"
			html = "<div class='container' id='#{self.ui_id}' #{style}>\n\n"
			@child.each { |control|
				html += control.to_html
			}
			html += "\n</div>\n"
			return html
		end
    
    # @since 2.5.0
    def add_controls_to_webdialog
      @child.each { |control|
        @window.add_control_to_webdialog(control)
        if control.is_a?( ContainerElement )
          control.add_controls_to_webdialog
        end
      }
    end
    
    # @since 2.5.0
    def get_control_by_ui_id(ui_id)
      @child.each { |control|
				return control if control.ui_id == ui_id
        if control.is_a?( ContainerElement )
          r = control.get_control_by_ui_id( ui_id )
          return r if r
        end
			}
      nil
    end
    
	end
	
	
	# @since 2.4.0
	class Container < Control
		include ContainerElement
	end
	
	
	# @since 2.4.0
  # (!) Create ToolbarButton class
	class Button < Control
		# @since 2.4.0
		attr_accessor( :caption, :icon )
		
		# @since 2.4.0
		def initialize(caption, &on_click)
			super
			@caption = caption
			#@on_click = on_click
      add_event(:click, on_click)
		end
    
    # @since 2.5.0
    def custom_properties
      prop = TT::JSON.new
      prop['caption'] = @caption
      prop['icon'] = @icon if @icon
      prop
    end
		
		# @since 2.4.0
		#def window=(webdialog)
		#	webdialog.add_action_callback(self.ui_id) { |dialog, params|
		#		puts ">> #{self.ui_id}.on_click(#{params})"
		#		@on_click.call(params)
		#	}
		#end
		
		# @since 2.4.0
		def to_html
			style = ( (css = self.css).nil? ) ? '' : " style='#{css}'"
			html = %{<span class='button normal' id='#{self.ui_id}' #{style} onclick='window.location="skp:#{self.ui_id}"'>#{self.caption}</span>\n}
		end
	end
	
	
	# @since 2.4.0
	class Checkbox < Control
		# @since 2.4.0
		attr_accessor( :checked )
		
		# @since 2.4.0
		def initialize(label)
			super
			@label = label
		end
	end
	
	
	# @since 2.4.0
	#class OptionList < Control
	class Listbox < Control
    # @since 2.5.0
		attr_reader( :value, :items, :size, :multiple, :label )
		
    # (!) Remove Label - should be induvidual UI element.
    #
		# @since 2.5.0
		def initialize(list = nil)
			super
			@value = nil
      @items = ( list.is_a?(Array) ) ? list : [] # (?) Hash instead?
      @label = nil
      @size = nil
      @multiple = false
		end
    
    # @since 2.5.0
    def custom_properties
      prop = TT::JSON.new
      prop['items'] = @items
      prop['size']  = @size if @size
      prop['label'] = @label if @label
      prop['multiple'] = @multiple if @multiple
      prop
    end
    
    # (?) Index argument?
    # @since 2.5.0
    def on_change(&block)
      add_event(:change, block)
    end
    
    # @since 2.5.0
    def add_item(string)
      @items << string
    end
    
    # @since 2.5.0
    # (!) Remove this. Make Bezier Surface create this.
    def label=(string)
      @label = string
    end
    
	end
	
	
	# @since 2.4.0
	class Textbox < Control
		# @since 2.4.0
		attr_accessor( :value )
		
		# @since 2.4.0
		def initialize(value)
			super
			@value = value
		end
		
		# @since 2.4.0
		def to_html
			style = ( (css = self.css).nil? ) ? '' : " style='#{css}'"
			html = "<input type='text' id='#{self.ui_id}' value='#{self.value}' #{style} />\n"
		end
	end
	
	
	# @since 2.4.0
	class Label < Control
		# @since 2.4.0
		attr_accessor( :caption, :align_right )
		
		# @since 2.4.0
		def initialize(caption, align_right=true)
			super
			@caption = caption
			@align_right = align_right
		end
		
		# @since 2.4.0
		def to_html
			style = ( (css = self.css).nil? ) ? '' : " style='#{css}'"
			html = "<label #{style}>#{self.caption}</label>\n"
		end
	end

end # module TT::GUI