/*
 * Common methods for webdialogs.
 *
 * Requires jQuery
 *
 * Javascript Namespace: http://www.dustindiaz.com/namespace-your-javascript/
 * http://www.crockford.com/javascript/private.html
 */


 /* Sketchup namespace */
var Sketchup = function() {
  return {
  
  
    /* Relay events back to the webdialog.
     */
    callback : function(controlID, event, args) {
      if ( args === undefined ) {
        params = controlID+'||'+event;
      }
      else {
        params = controlID+'||'+event+'||'+args.join(',');
      }
      window.location = 'skp:Event_Callback@'+params;
    }
    
    
  };
  
}(); // SketchUp


 /* Webdialog namespace */
var Webdialog = function() {
  return {
  
  
    /* Returns an array of the viewports' size.
     */
    get_client_size : function() {
      return [ $(window).width(), $(window).height() ];
    },
    
    
    /* Returns the HTML for the given jQuery selector.
     */
    get_html : function(selector) {
      return $(selector).html();
    },
    
    
    /* Returns the HTML for the given jQuery selector.
     */
    get_text : function(selector) {
      return $(selector).text();
    }
    
    
  };
  
}(); // WebDialog


/* Bridge Namespace */
var Bridge = function() {
  return {
    
    
    /* Resets the Ruby bridge.
     * (private ?)
     */
    escape_string : function( value ) {
      return value.replace('\\', '\\\\').replace("'", "\'");
    },
    
    
    /* Resets the Ruby bridge.
     */
    execute : function( code_string ) {
      // CleanUp <SCRIPT></SCRIPT> elements which Ruby UI::WebDialog.execute_script
      // leaves behind.
      $('body script').detach();
      // Execute the JavaScript code and put the return value back into the bridge.
      // (!) Catch error.
      Bridge.return_ruby( eval(code_string) );
    },
    
    
    
    /* Resets the Ruby bridge.
     */
    reset : function() {
      $('#RUBY_bridge').val( '' );
    },
    
    
    /* Resets the Ruby bridge.
     */
    return_ruby : function( value ) {
      $('#RUBY_bridge').val( Bridge.value_to_ruby(value) );
    },
    
    
    /* Resets the Ruby bridge.
     *
     * TODO:
     * * JSON
     * * ...
     *
     * (private ?)
     */
    value_to_ruby : function( value ) {
      var ruby_string = '';
      switch ( $.type( value ) ) {
        case 'boolean':
          ruby_string = value.toString();
          break;
        case 'number':
          if ( isNaN( value ) ) {
            ruby_string = '0.0/0.0';
          } else if ( isFinite( value ) ) {
            ruby_string = value.toString();
          } else {
            // Infinite
            ruby_string = ( value > 0 ) ? '1.0/0.0' : '-1.0/0.0';
          }
          break;
        case 'string':
          ruby_string = "'" + Bridge.escape_string( value ) + "'";
          break;
        case 'null':
        case 'undefined':
          ruby_string = 'nil';
          break;
        case 'array':
          ruby_values = $.map(value, function(value, index) { 
            return Bridge.value_to_ruby( value );
          });
          ruby_string = '[' + ruby_values.join(',') + ']';
          break;
        case 'date':
          ruby_string = 'Time.at(' + value.getTime() + ')';
          break;
        case 'regexp':
          ruby_string = "'<REGEXP>'";
          break;
        case 'function':
          ruby_string = "'<FUNCTION>'";
          break;
        case 'object':
          // (!) JSON
          ruby_string = "'<OBJECT>'";
          break;
      }
      return ruby_string;
    }
    
    
  };
  
}(); // Bridge


/* UI namespace */
var UI = function() {
  return {
  
  
    init : function() {
      // Ruby Bridge
      bridge = $('<input id="RUBY_bridge" name="RUBY_bridge" type="hidden" />');
      $('body').append( bridge );
      // Focus property
      UI.add_focus_property()
      // Buttons
      $('.button').live('mousedown', function() { $(this).addClass('pressed'); return false; });
      $('.button').live('mouseup', function() { $(this).removeClass('pressed'); return false; });
      // Ready Event
      window.location = 'skp:Window_Ready';
    },
    
    
    /* Toggle the content DIV of the clicked element this function is attached to.
     * Attach this event to the Click event of its sibling;
     * <div>
     *   <h2>Section Header</h2>
     *   <div>Content DIV which is toggled.</div>
     * </div>
     */
    toggle_content : function() {
      $(this).siblings('div').slideToggle('fast');
    },
    
    
    // Ensure links are opened in the default browser.
    redirect_links : function() {
      $('a.url').live('click', function()
      {
        window.location = 'skp:url@' + this.href;
        return false;
      } );
    },
    
    
    /* Loops over all input elements and ensure that they get an .focus class
     * added upon focus and remove it when it loses focus. This is a workaround
     * for IE7's lack of :hover support.
     */
    add_focus_property : function() {
      $('input').live('focusin', function () {
        $(this).addClass('focus');
      });
      $('input').live('focusout', function () {
        $(this).removeClass('focus');
      });
    },
    
    
    /* Adds a control to the window.
     */
    add_control : function(properties) {
      switch ( properties.type )
      {
      case 'TT::GUI::Button':
        UI.add_button( properties );
        break;
      case 'TT::GUI::Listbox':
        UI.add_list( properties );
        break;
      default:
        alert('Invalid Control Type.')
        break;
      }
    },
    
    
    /* Adds a button.
     */
    add_button : function(properties) {
      var $parent = get_parent( properties );
      var $button = $('<div class="button"></div>')
      $button.id = properties.id
      if ( 'icon' in properties )
      {
        $button.attr( 'title', properties.caption );
        var $icon = $('<img />').appendTo( $button );
        $icon.attr('src', properties.icon);
        $icon.attr('alt', 'N/A');
      }
      else
      {
        $button.text( properties.caption );
      }
      $button.click( function(){
        Sketchup.callback( $button.id, 'click' );
      } );
      $button.appendTo( $parent );
    },
    
    
    /* Adds a list.
     * (!) Remove label - make into separate UI object.
     */
    add_list : function(properties) {
      var $parent = get_parent( properties );
      var $control = $('<div class="list"></div>').appendTo( $parent );
      $control.id = properties.id
      var list_control_id = properties.id+'_list';
      if ( 'label' in properties ) {
        var $label = $('<label/>');
        $label.attr( 'for', list_control_id );
        $label.text( properties.label );
        $label.appendTo( $control );
      }
      if ( 'size' in properties ) {
        $control.attr( 'size', properties.size )
      }
      if ( 'multiple' in properties && properties.multiple ) {
        $list.attr( 'multiple', 'multiple' );
      }
      var $list = $('<select/>')
      $list.id = list_control_id;
      if ( 'items' in properties ) {
        var items = properties.items;
        for ( i in items ) {
          $item = $('<option/>');
          $item.text( items[i] );
          $item.attr( 'value', items[i] );
          $item.appendTo( $list );
        }
      }
      $list.appendTo( $control );
      $list.change( function(){
        var $this = $(this);
        var args = [ $this.val() ];
        Sketchup.callback( $control.id, 'change', args );
      } );
    }
    
    
  };
  
  /* PRIVATE */
  
  /* Returns the parent object.
   */
  function get_parent(properties) {
    if ( 'parent' in properties ) {
      return $( '#' + properties.parent );
    }
    else {
      return $( 'body' );
    }
  }
  
}(); // UI
$(document).ready(UI.init);



/*
 * Natural Sort algorithm for Javascript - Version 0.4 - Released under MIT license
 * Author: Jim Palmer (based on chunking idea from Dave Koelle)
 * Contributors: Mike Grier (mgrier.com), Clint Priest, Kyle Adams
 */
function naturalSort(a, b) {
  // setup temp-scope variables for comparison evauluation
  var re = /(^[0-9]+\.?[0-9]*[df]?e?[0-9]?$|^0x[0-9a-f]+$|[0-9]+)/gi,
    sre = /(^[ ]*|[ ]*$)/g,
    hre = /^0x[0-9a-f]+$/i,
    ore = /^0/,
    // convert all to strings and trim()
    x = a.toString().replace(sre, '') || '',
    y = b.toString().replace(sre, '') || '',
    // chunk/tokenize
    xN = x.replace(re, '\0$1\0').replace(/\0$/,'').replace(/^\0/,'').split('\0'),
    yN = y.replace(re, '\0$1\0').replace(/\0$/,'').replace(/^\0/,'').split('\0'),
    // hex or date detection
    xD = parseInt(x.match(hre)) || (new Date(x)).getTime(),
    yD = parseInt(y.match(hre)) || xD && (new Date(y)).getTime() || null;
  // natural sorting of hex or dates - prevent '1.2.3' valid date
  if ( y.indexOf('.') < 0 && yD )
    if ( xD < yD ) return -1;
    else if ( xD > yD )	return 1;
  // natural sorting through split numeric strings and default strings
  for(var cLoc=0, numS=Math.max(xN.length, yN.length); cLoc < numS; cLoc++) {
    // find floats not starting with '0', string or 0 if not defined (Clint Priest)
    oFxNcL = !(xN[cLoc] || '').match(ore) && parseFloat(xN[cLoc]) || xN[cLoc] || 0;
    oFyNcL = !(yN[cLoc] || '').match(ore) && parseFloat(yN[cLoc]) || yN[cLoc] || 0;
    // handle numeric vs string comparison - number < string - (Kyle Adams)
    if (isNaN(oFxNcL) !== isNaN(oFyNcL)) return (isNaN(oFxNcL)) ? 1 : -1; 
    // rely on string comparison if different types - i.e. '02' < 2 != '02' < '2'
    else if (typeof oFxNcL !== typeof oFyNcL) {
      oFxNcL += ''; 
      oFyNcL += ''; 
    }
    if (oFxNcL < oFyNcL) return -1;
    if (oFxNcL > oFyNcL) return 1;
  }
  return 0;
}